<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns="http://www.ascc.net/xml/schematron"
                xmlns:g="http://genericode.org/2005/ns/CodeList/0.3/"
             xmlns:cl="urn:oasis:names:specification:ubl:schema:xsd:Code-1.0"
             xmlns:clc="urn:oasis:names:tc:ubl:schema:CodeList-Constraints-0.4"
                version="1.0"
                exclude-result-prefixes="g clc xsl">
<!--
     Support for an implementation of the UBL Code List Value Validation 
     Methodology.

     This fragment determines the names of meta data information items
     related to code list information items.

  $Id: UBL-Metadata-0.6.xsl,v 1.1 2006/06/17 00:06:10 G. Ken Holman Exp $

Universal Business Language (UBL)

Copyright (C) OASIS Open (2006). All Rights Reserved.

This document and translations of it may be copied and furnished to others, and
derivative works that comment on or otherwise explain it or assist in its
implementation may be prepared, copied, published and distributed, in whole or
in part, without restriction of any kind, provided that the above copyright
notice and this paragraph are included on all such copies and derivative works.
However, this document itself may not be modified in any way, such as by
removing the copyright notice or references to OASIS, except as needed for the
purpose of developing OASIS specifications, in which case the procedures for
copyrights defined in the OASIS Intellectual Property Rights document must be
followed, or as required to translate it into languages other than English.

The limited permissions granted above are perpetual and will not be revoked by
OASIS or its successors or assigns.

This document and the information contained herein is provided on an "AS IS"
basis and OASIS DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO ANY WARRANTY THAT THE USE OF THE INFORMATION HEREIN WILL NOT
INFRINGE ANY RIGHTS OR ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR
A PARTICULAR PURPOSE.

-->

<!--========================================================================-->
<!--for a given information item, engage the checking of various metadata
    properties supported by the external code list implementation-->

<xsl:template name="document-model-metadata">
  <xsl:param name="item"/>
  <!--determine name stem based on structure of item name-->
  <xsl:variable name="namestem">
    <xsl:choose>
      <xsl:when test="starts-with($item,'@') and contains($item,'ID') and
                      substring-after($item,'ID')=''">
        <!--no metadata for attributes ending in 'ID'-->
      </xsl:when>
      <xsl:when test="starts-with($item,'@') and contains($item,'Code') and
                      substring-after($item,'Code')=''">
        <!--no metadata for attributes ending in 'Code'-->
      </xsl:when>
      <xsl:when test="contains($item,'ID') and
                      substring-after($item,'ID')=''">
        <xsl:text>@scheme</xsl:text>
      </xsl:when>
      <xsl:when test="contains($item,'Code') and
                      substring-after($item,'Code')=''">
        <xsl:text>@list</xsl:text>
      </xsl:when>
    </xsl:choose>
  </xsl:variable>
  <xsl:if test="string($namestem)">
    <!--for each document model address, check associated code list property-->
    <xsl:call-template name="check-list-property">
      <xsl:with-param name="address" select="concat($namestem,'Name')"/>
      <xsl:with-param name="property" select="'Name'"/>
    </xsl:call-template>
    <xsl:call-template name="check-list-property">
      <xsl:with-param name="address" select="concat($namestem,'ID')"/>
      <xsl:with-param name="property" select="'ID'"/>
    </xsl:call-template>
    <xsl:call-template name="check-list-property">
      <xsl:with-param name="address" select="concat($namestem,'VersionID')"/>
      <xsl:with-param name="property" select="'VersionID'"/>
    </xsl:call-template>
    <xsl:call-template name="check-list-property">
      <xsl:with-param name="address" select="concat($namestem,'AgencyID')"/>
      <xsl:with-param name="property" select="'AgencyID'"/>
    </xsl:call-template>
    <xsl:call-template name="check-list-property">
      <xsl:with-param name="address" select="concat($namestem,'AgencyName')"/>
      <xsl:with-param name="property" select="'AgencyName'"/>
    </xsl:call-template>
    <!--special cases for the URI values-->
    <xsl:if test="contains($item,'ID') and substring-after($item,'ID')=''">
      <xsl:call-template name="check-list-property">
        <xsl:with-param name="address" select="concat($namestem,'DataURI')"/>
        <xsl:with-param name="property" select="'LocationURI'"/>
      </xsl:call-template>
      <xsl:call-template name="check-list-property">
        <xsl:with-param name="address" select="concat($namestem,'URI')"/>
        <xsl:with-param name="property" select="'SchemeURI'"/>
      </xsl:call-template>
    </xsl:if>
    <xsl:if test="contains($item,'Code') and substring-after($item,'Code')=''">
      <xsl:call-template name="check-list-property">
        <xsl:with-param name="address" select="concat($namestem,'SchemeURI')"/>
        <xsl:with-param name="property" select="'SchemeURI'"/>
      </xsl:call-template>
      <xsl:call-template name="check-list-property">
        <xsl:with-param name="address" select="concat($namestem,'URI')"/>
        <xsl:with-param name="property" select="'LocationURI'"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:if>
</xsl:template>

</xsl:stylesheet>