<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns="http://purl.oclc.org/dsdl/schematron"
                xmlns:g="http://genericode.org/2006/ns/CodeList/0.4/"
               xmlns:cl="urn:oasis:names:specification:ubl:schema:xsd:Code-1.0"
                version="1.0"
                exclude-result-prefixes="g cl">
<!--
     Support for an implementation of the UBL Code List Value Validation 
     Methodology.

     This fragment supports external code list expressions of code list values
     written using genericode 0.4.

  $Id: genericode-CodeList-0.6.xsl,v 1.1 2006/06/18 16:56:04 G. Ken Holman Exp $

Universal Business Language (UBL)

Copyright (C) OASIS Open (2006). All Rights Reserved.

This document and translations of it may be copied and furnished to others, and
derivative works that comment on or otherwise explain it or assist in its
implementation may be prepared, copied, published and distributed, in whole or
in part, without restriction of any kind, provided that the above copyright
notice and this paragraph are included on all such copies and derivative works.
However, this document itself may not be modified in any way, such as by
removing the copyright notice or references to OASIS, except as needed for the
purpose of developing OASIS specifications, in which case the procedures for
copyrights defined in the OASIS Intellectual Property Rights document must be
followed, or as required to translate it into languages other than English.

The limited permissions granted above are perpetual and will not be revoked by
OASIS or its successors or assigns.

This document and the information contained herein is provided on an "AS IS"
basis and OASIS DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO ANY WARRANTY THAT THE USE OF THE INFORMATION HEREIN WILL NOT
INFRINGE ANY RIGHTS OR ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR
A PARTICULAR PURPOSE.

-->

<!--========================================================================-->
<!--determine the presence of a document with the expected document element-->
<!--returns the string of an error message if there is a problem-->

<xsl:template name="code-list-confirm-instance">
  <xsl:param name="uri"/>
  <xsl:param name="doc"/>
  <xsl:if test="not($doc/g:CodeList)">
    <xsl:text>Resource not pointing to {</xsl:text>
    <xsl:value-of select="document('')/*/namespace::g"/>
    <xsl:text>}CodeList: </xsl:text>
    <xsl:value-of select="concat($uri,'&#xa;  ')"/>
    <xsl:call-template name="report-current-node-context"/>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<!--build Schematron <rule> elements using the selection of available values-->
<!--note the liberal use of newline sequences in the result using <xsl:text>
    in order to make the end result legible to a reader; these have no effect
    on the well-formedness of the result or the Schematron validation-->

<xsl:template name="code-list-make-rule">
  <xsl:param name="item"/>
  <xsl:param name="context"/>
  <xsl:param name="value"/>
  <xsl:param name="codes-identifier"/>
  <xsl:param name="assoc-uri"/>
  <xsl:param name="codelists-referenced"/>
  <xsl:variable name="codes" select="document($codelists-referenced/@uri,.)/
          g:CodeList/SimpleCodeList/Row/Value[@ColumnRef='code']/SimpleValue"/>
  <!--only check an item if there are codes against which to compare-->
  <xsl:if test="$codes">
    <xsl:variable name="bad-in-codes">
      <xsl:for-each select="$codes">
        <xsl:if test="normalize-space(.)!=.">y</xsl:if>
      </xsl:for-each>
    </xsl:variable>
    <xsl:text>
</xsl:text>
    <rule context="{$context}"><xsl:text>
  </xsl:text>
      <!--indicate values that must be true-->
      <assert>
        <xsl:attribute name="test">
          <xsl:for-each select="document($codelists-referenced/@uri,.)/
                                g:CodeList
                                [SimpleCodeList/Row/Value[@ColumnRef='code']/
                                 SimpleValue]">
            <xsl:if test="position()>1"> or </xsl:if>
            <xsl:text>( ( </xsl:text>
            <xsl:variable name="metadata">
              <!--check properties based on item's name stem-->
              <xsl:call-template name="build-metadata-comparison-string">
                <xsl:with-param name="item" select="$item"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:if test="string($metadata)">
              <!--only include metadata if there are comparisons-->
              <xsl:value-of select="concat($metadata,' ) and ( ')"/>
            </xsl:if>
            <!--values of information item for this particular context-->
            <xsl:choose>
              <xsl:when test="$value='token' and not(string($bad-in-codes))">
                <xsl:text>contains( ' </xsl:text>
                <xsl:for-each select="SimpleCodeList/Row/
                                      Value[@ColumnRef='code']/SimpleValue">
                  <xsl:if test="position()>1"><xsl:text> </xsl:text></xsl:if>
                  <xsl:value-of select="."/>
                </xsl:for-each>
                <xsl:text> ',concat(' ',normalize-space(.),' ') )</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:for-each select="SimpleCodeList/Row/
                                      Value[@ColumnRef='code']/SimpleValue">
                  <xsl:if test="position()>1"> or </xsl:if>
                  <xsl:text/>. = '<xsl:value-of select="."/>'<xsl:text/>
                </xsl:for-each>
              </xsl:otherwise>
            </xsl:choose>
            <xsl:text> ) )</xsl:text>
          </xsl:for-each>
        </xsl:attribute>
        <!--message to report if the test fails-->
        <xsl:text/>Value supplied '<value-of select="."/>
        <xsl:text>' is unacceptable for codes identified by '</xsl:text>
        <xsl:value-of select="$codes-identifier"/>
        <xsl:if test="$assoc-uri">
          <xsl:text>' in association file with uri '</xsl:text>
          <xsl:value-of select="$assoc-uri"/>
        </xsl:if>
        <xsl:text>' in the context: </xsl:text>
        <xsl:value-of select="$context"/>
      </assert><xsl:text>
</xsl:text>
    </rule><xsl:text>
</xsl:text>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<!--
     Return the code list's metadata value string for the requested property.
     Return the empty string if no such property is available.

     genericode <Identification> child elements:

       property ID          ==  ShortName            
       property Name        ==  LongName             
       property VersionID   ==  Version              
       property SchemeURI   ==  CanonicalVersionUri  
       property LocationURI ==  AlternateFormatLocationUri
       property AgencyID    ==  Agency/Identifier
       property AgencyName  ==  Agency/LongName
-->

<xsl:template name="code-list-get-property-value">
  <xsl:param name="property"/>

  <xsl:variable name="property-value">
    <xsl:choose>
      <xsl:when test="$property='ID'">
        <xsl:value-of select="Identification/ShortName"/>
      </xsl:when>
      <xsl:when test="$property='Name'">
        <xsl:value-of select="Identification/LongName"/>
      </xsl:when>
      <xsl:when test="$property='VersionID'">
        <xsl:value-of select="Identification/Version"/>
      </xsl:when>
      <xsl:when test="$property='SchemeURI'">
        <xsl:value-of select="Identification/CanonicalVersionUri"/>
      </xsl:when>
      <xsl:when test="$property='LocationURI'">
        <xsl:value-of select="Identification/AlternateFormatLocationUri"/>
      </xsl:when>
      <xsl:when test="$property='AgencyID'">
        <xsl:value-of select="Identification/Agency/Identifier"/>
      </xsl:when>
      <xsl:when test="$property='AgencyName'">
        <xsl:value-of select="Identification/Agency/LongName"/>
      </xsl:when>
    </xsl:choose>
  </xsl:variable>
  <xsl:value-of select="normalize-space($property-value)"/>
</xsl:template>

</xsl:stylesheet>
